@@ -24,7 +24,7 @@
 #include <string.h>
 
 #include "pango-attributes.h"
-#include "pango-utils.h"
+#include "pango-impl-utils.h"
 
 struct _PangoAttrList
 {
@@ -472,7 +472,7 @@ pango_attr_size_new_internal (int size,
 
 /**
  * pango_attr_size_new:
- * @size: the font size, in #PANGO_SCALE<!-- -->ths of a point.
+ * @size: the font size, in %PANGO_SCALE<!-- -->ths of a point.
  * 
  * Create a new font-size attribute in fractional points.
  * 
@@ -487,7 +487,7 @@ pango_attr_size_new (int size)
 
 /**
  * pango_attr_size_new_absolute:
- * @size: the font size, in #PANGO_SCALE<!-- -->ths of a device unit.
+ * @size: the font size, in %PANGO_SCALE<!-- -->ths of a device unit.
  * 
  * Create a new font-size attribute in device units.
  * 
@@ -1840,13 +1840,11 @@ pango_attr_list_filter (PangoAttrList       *list,
   PangoAttrList *new = NULL;
   GSList *tmp_list;
   GSList *prev;
-  GSList *new_attrs;
   
   g_return_val_if_fail (list != NULL, NULL);
 
   tmp_list = list->attributes;
   prev = NULL;
-  new_attrs = NULL;
   while (tmp_list)
     {
       GSList *next = tmp_list->next;
