@@ -25,9 +25,8 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "pango-impl-utils.h"
-#include "pango-utils.h"
 #include "pango-font.h"
+#include "pango-impl-utils.h"
 
 #include <glib/gstdio.h>
 
@@ -460,7 +459,7 @@ read_config_file (const char *filename, gboolean enoent_error)
   GError *key_file_error = NULL;
   gchar **groups;
   gsize groups_count = 0;
-  gint group_index;
+  guint group_index;
   
   if (!g_key_file_load_from_file(key_file,filename, 0, &key_file_error))
     {
@@ -489,7 +488,7 @@ read_config_file (const char *filename, gboolean enoent_error)
       
       if (keys)
 	{
-	  gint key_index;
+	  guint key_index;
 
 	  for (key_index = 0; key_index < keys_count; key_index++)
 	    {
@@ -694,7 +693,7 @@ pango_parse_style (const char *str,
 /**
  * pango_parse_variant:
  * @str: a string to parse.
- * @style: a #PangoVariant to store the result in.
+ * @variant: a #PangoVariant to store the result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
  * 
  * Parses a font variant. The allowed values are "normal"
@@ -740,7 +739,7 @@ pango_parse_variant (const char   *str,
 /**
  * pango_parse_weight:
  * @str: a string to parse.
- * @style: a #PangoWeight to store the result in.
+ * @weight: a #PangoWeight to store the result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
  * 
  * Parses a font weight. The allowed values are "heavy",
@@ -836,7 +835,7 @@ pango_parse_weight (const char  *str,
 /**
  * pango_parse_stretch:
  * @str: a string to parse.
- * @style: a #PangoStretch to store the result in.
+ * @stretch: a #PangoStretch to store the result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
  * 
  * Parses a font stretch. The allowed values are 
@@ -1172,15 +1171,17 @@ lang_equal (gconstpointer v1,
   const guchar *p1 = v1;
   const guchar *p2 = v2;
 
-  while (*p2)
+  while (*p1 && *p2)
     {
-      guchar value = canon_map[*p2];
-      if (value && value != *p1++)
-	return FALSE;
-      p2++;
+      guchar c1 = canon_map[*p1];
+      guchar c2 = canon_map[*p2];
+      if (!c1) { p1++; continue; }
+      if (!c2) { p2++; continue; }
+      if (c1 != c2) return FALSE;
+      p1++, p2++;
     }
 
-  return (*p1 == '\0');
+  return (*p1 == *p2);
 }
 
 static guint
@@ -1247,12 +1248,14 @@ pango_language_from_string (const char *language)
   int len;
   char *p;
 
-  if (!hash)
+  if (G_UNLIKELY (!hash))
     hash = g_hash_table_new (lang_hash, lang_equal);
-
-  result = g_hash_table_lookup (hash, language);
-  if (result)
-    return (PangoLanguage *)result;
+  else
+    {
+      result = g_hash_table_lookup (hash, language);
+      if (result)
+	return (PangoLanguage *)result;
+    }
 
   len = strlen (language);
   result = g_malloc (len + 1);
@@ -1317,7 +1320,7 @@ pango_language_matches (PangoLanguage *language,
 }
 
 typedef struct {
-  const char *lang;
+  const char lang[4];
   const char *str;
 } LangInfo;
 
@@ -1401,9 +1404,29 @@ pango_language_get_sample_string (PangoLanguage *language)
   return result;
 }
 
+/**
+ * pango_log2vis_get_embedding_levels:
+ * @text:      the text to itemize.
+ * @length:    the number of bytes (not characters) to process, or -1
+ *             if @text is nul-terminated and the legnth should be calculated.
+ * @pbase_dir: input base direction, and output resolved direction.
+ *
+ * This will return the bidirectional embedding levels of the input paragraph
+ * as defined by the Unicode Bidirectional Algorithm available at:
+ *
+ *   http://www.unicode.org/reports/tr9/
+ *
+ * If the input base direction is a weak direction, the direction of the
+ * characters in the text will determine the final resolved direction.
+ *
+ * Return value: a newly allocated array of embedding levels, one item per
+ *               character (not byte), that should be freed using g_free.
+ *
+ * Since: 1.4
+ */
 guint8 *
-pango_log2vis_get_embedding_levels (const gchar    *str,
-				    int             bytelen,
+pango_log2vis_get_embedding_levels (const gchar    *text,
+				    int             length,
 				    PangoDirection *pbase_dir)
 {
   FriBidiCharType fribidi_base_dir;
@@ -1433,15 +1456,15 @@ pango_log2vis_get_embedding_levels (const gchar    *str,
 
 #ifdef FRIBIDI_HAVE_UTF8
   {
-    if (bytelen < 0)
-      bytelen = strlen (str);
-    embedding_levels_list = fribidi_log2vis_get_embedding_levels_new_utf8 (str, bytelen, &fribidi_base_dir);
+    if (length < 0)
+      length = strlen (text);
+    embedding_levels_list = fribidi_log2vis_get_embedding_levels_new_utf8 (text, length, &fribidi_base_dir);
   }
 #else
   {
     gunichar *text_ucs4;
     int n_chars;
-    text_ucs4 = g_utf8_to_ucs4_fast (str, bytelen, &n_chars);
+    text_ucs4 = g_utf8_to_ucs4_fast (text, length, &n_chars);
     embedding_levels_list = g_new (guint8, n_chars);
     fribidi_log2vis_get_embedding_levels ((FriBidiChar*)text_ucs4, n_chars,
 					  &fribidi_base_dir,
@@ -1558,7 +1581,6 @@ read_alias_file (const char *filename)
 
   while (pango_read_line (file, line_buffer))
     {
-      gboolean empty = FALSE;
       gboolean append = FALSE;
       line++;
 
@@ -1585,22 +1607,15 @@ read_alias_file (const char *filename)
 	  goto error;
 	}
       
-      if (!pango_skip_space (&pos))
+      if (!pango_scan_string (&pos, tmp_buffer2))
 	{
-	  empty = TRUE;
+	  errstring = g_strdup ("Error parsing value string");
+	  goto error;
 	}
-      else
+      if (pango_skip_space (&pos))
 	{
-	  if (!pango_scan_string (&pos, tmp_buffer2))
-	    {
-	      errstring = g_strdup ("Error parsing value string");
-	      goto error;
-	    }
-	  if (pango_skip_space (&pos))
-	    {
-	      errstring = g_strdup ("Junk after value string");
-	      goto error;
-	    }
+	  errstring = g_strdup ("Junk after value string");
+	  goto error;
 	}
 
       alias_key.alias = g_ascii_strdown (tmp_buffer1->str, -1);
@@ -1698,13 +1713,11 @@ pango_load_aliases (void)
  * @fontname: an ascii string
  * @families: will be set to an array of font family names.
  *    this array is owned by pango and should not be freed.
+ * @n_families: will be set to the length of the @families array.
  *
- * Look up all user defined aliases for the alias #fontname.
- * The resulting font family names will be stored in #families,
- * and the number of families will be returned.
- * 
- * Return value: the number of font famillies stored in the #families argument.
- *   This value is owned by Pango and must not be freed.
+ * Look up all user defined aliases for the alias @fontname.
+ * The resulting font family names will be stored in @families,
+ * and the number of families in @n_families.
  **/
 void
 pango_lookup_aliases (const char   *fontname,
@@ -1786,37 +1799,57 @@ gboolean
 pango_is_zero_width (gunichar ch)
 {
 /* Zero Width characters:
- *
- *  00AD  SOFT HYPHEN
- *  034F  COMBINING GRAPHEME JOINER
  *
  *  200B  ZERO WIDTH SPACE
  *  200C  ZERO WIDTH NON-JOINER
  *  200D  ZERO WIDTH JOINER
  *  200E  LEFT-TO-RIGHT MARK
  *  200F  RIGHT-TO-LEFT MARK
- *
  *  2028  LINE SEPARATOR
- *
  *  202A  LEFT-TO-RIGHT EMBEDDING
  *  202B  RIGHT-TO-LEFT EMBEDDING
  *  202C  POP DIRECTIONAL FORMATTING
  *  202D  LEFT-TO-RIGHT OVERRIDE
  *  202E  RIGHT-TO-LEFT OVERRIDE
- *
- *  2060  WORD JOINER
- *  2061  FUNCTION APPLICATION
- *  2062  INVISIBLE TIMES
- *  2063  INVISIBLE SEPARATOR
- *
  *  FEFF  ZERO WIDTH NO-BREAK SPACE
  */
-  return ((ch & ~(gunichar)0x007F) == 0x2000 && (
+  return ((ch & ~(gunichar)0x003F) == 0x2000 && (
 		(ch >= 0x200B && ch <= 0x200F) ||
-		(ch >= 0x202A && ch <= 0x202E) ||
-		(ch >= 0x2060 && ch <= 0x2063) ||
-		(ch == 0x2028)
-	 )) || G_UNLIKELY (ch == 0x00AD
-			|| ch == 0x034F
-			|| ch == 0xFEFF);
+		ch == 0x2028 ||
+		(ch >= 0x202A && ch <= 0x202E)
+	 )) || ch == 0xFEFF;
+}
+
+/**
+ * pango_quantize_line_geometry:
+ * @thickness: pointer to the thickness of a line, in Pango scaled units
+ * @position: corresponding position
+ *
+ * Quantizes the thickness and position of a line, typically an
+ * underline or strikethrough, to whole device pixels, that is integer
+ * multiples of %PANGO_SCALE. The purpose of this function is to avoid
+ * such lines looking blurry.
+ *
+ * Since: 1.12
+ */
+void
+pango_quantize_line_geometry (int *thickness,
+			      int *position)
+{
+  int thickness_pixels = (*thickness + PANGO_SCALE / 2) / PANGO_SCALE;
+  if (thickness_pixels == 0)
+    thickness_pixels = 1;
+  
+  if (thickness_pixels & 1)
+    {
+      int new_center = ((*position - *thickness / 2) & ~(PANGO_SCALE - 1)) + PANGO_SCALE / 2;
+      *position = new_center + (PANGO_SCALE * thickness_pixels) / 2;
+    }
+  else
+    {
+      int new_center = ((*position - *thickness / 2 + PANGO_SCALE / 2) & ~(PANGO_SCALE - 1));
+      *position = new_center + (PANGO_SCALE * thickness_pixels) / 2;
+    }
+
+  *thickness = thickness_pixels * PANGO_SCALE;
 }
