@@ -26,7 +26,7 @@
 #include <string.h>
 
 #include "pango-fontmap.h"
-#include "pango-utils.h"
+#include "pango-impl-utils.h"
 #include "pangocairo-private.h"
 #include "pangocairo-win32.h"
 
@@ -212,16 +212,7 @@ compute_glyph_extents (PangoFont        *font,
   logical_rect->width = 0;
   logical_rect->height = (font_extents.ascent + font_extents.descent) * PANGO_SCALE;
 
-  if (glyph == PANGO_GLYPH_EMPTY)
-    {
-      /* already initialized above */
-    }
-  else if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
-    {
-      /* space for the hex box */
-      _pango_cairo_get_glyph_extents_missing(cfont, glyph, ink_rect, logical_rect);
-    }
-  else
+  if (glyph != PANGO_GLYPH_EMPTY)
     {
       cairo_glyph.index = glyph;
       cairo_glyph.x = 0;
@@ -277,52 +268,6 @@ pango_cairo_win32_font_get_glyph_extents (PangoFont        *font,
     *logical_rect = info->logical_rect;
 }
 
-static void
-quantize_position (int *thickness,
-		   int *position)
-{
-  int thickness_pixels = (*thickness + PANGO_SCALE / 2) / PANGO_SCALE;
-  if (thickness_pixels == 0)
-    thickness_pixels = 1;
-  
-  if (thickness_pixels & 1)
-    {
-      int new_center = ((*position - *thickness / 2) & ~(PANGO_SCALE - 1)) + PANGO_SCALE / 2;
-      *position = new_center + (PANGO_SCALE * thickness_pixels) / 2;
-    }
-  else
-    {
-      int new_center = ((*position - *thickness / 2 + PANGO_SCALE / 2) & ~(PANGO_SCALE - 1));
-      *position = new_center + (PANGO_SCALE * thickness_pixels) / 2;
-    }
-
-  *thickness = thickness_pixels * PANGO_SCALE;
-}
-
-static int
-max_glyph_width (PangoLayout *layout)
-{
-  int max_width = 0;
-  GSList *l, *r;
-
-  for (l = pango_layout_get_lines (layout); l; l = l->next)
-    {
-      PangoLayoutLine *line = l->data;
-
-      for (r = line->runs; r; r = r->next)
-        {
-          PangoGlyphString *glyphs = ((PangoGlyphItem *)r->data)->glyphs;
-          int i;
-
-          for (i = 0; i < glyphs->num_glyphs; i++)
-            if (glyphs->glyphs[i].geometry.width > max_width)
-              max_width = glyphs->glyphs[i].geometry.width;
-        }
-    }
-
-  return max_width;
-}
-
 static PangoFontMetrics *
 create_metrics_for_context (PangoFont    *font,
 			    PangoContext *context)
@@ -357,8 +302,10 @@ create_metrics_for_context (PangoFont    *font,
   metrics->strikethrough_thickness = metrics->underline_thickness;
   metrics->strikethrough_position = height / 4;
 
-  quantize_position (&metrics->underline_thickness, &metrics->underline_position);
-  quantize_position (&metrics->strikethrough_thickness, &metrics->strikethrough_position);
+  pango_quantize_line_geometry (&metrics->underline_thickness,
+				&metrics->underline_position);
+  pango_quantize_line_geometry (&metrics->strikethrough_thickness,
+				&metrics->strikethrough_position);
 
   layout = pango_layout_new (context);
   font_desc = pango_font_describe (font);
@@ -369,7 +316,9 @@ create_metrics_for_context (PangoFont    *font,
   metrics->approximate_char_width = extents.width / g_utf8_strlen (sample_str, -1);
 
   pango_layout_set_text (layout, "0123456789", -1);
-  metrics->approximate_digit_width = max_glyph_width (layout);
+  pango_layout_get_extents (layout, NULL, &extents);
+
+  metrics->approximate_digit_width = extents.width / 10;
 
   pango_font_description_free (font_desc);
   g_object_unref (layout);
