@@ -1,3 +1,25 @@
+/* Pango
+ * gen-script-for-lang.c: Utility program to generate pango-script-lang-table.h
+ *
+ * Copyright (C) 2003 Red Hat, Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+#include <config.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -19,7 +41,7 @@ static void scripts_for_file (const char *base_dir,
 			      const char *file_part,
 			      LangInfo   *info);
 
-const char *get_script_name (PangoScript script)
+static const char *get_script_name (PangoScript script)
 {
   static GEnumClass *class = NULL;
   GEnumValue *value;
@@ -32,7 +54,8 @@ const char *get_script_name (PangoScript script)
   return value->value_name;
 }
 
-int fail (char *format, ...)
+static void fail (const char *format, ...) G_GNUC_PRINTF (1, 2) G_GNUC_NORETURN;
+static void fail (const char *format, ...)
 {
   va_list vap;
   
@@ -43,7 +66,7 @@ int fail (char *format, ...)
   exit (1);
 }
 
-gboolean scan_hex (const char **str, gunichar *result)
+static gboolean scan_hex (const char **str, gunichar *result)
 {
   const char *end;
 
@@ -55,14 +78,6 @@ gboolean scan_hex (const char **str, gunichar *result)
   return TRUE;
 }
 
-void warn_mismatch (const char *file_part,
-		    PangoScript script1,
-		    PangoScript script2)
-{
-  g_printerr ("%s has characters for both %s and %s\n",
-	      file_part, get_script_name (script1), get_script_name (script2));
-}
-
 static void
 scripts_for_line (const char  *base_dir,
 		  const char  *file_part,
@@ -151,11 +166,11 @@ scripts_for_file (const char *base_dir,
   GIOStatus status = G_IO_STATUS_NORMAL;
 
   if (!channel)
-    fail ("Error opening '%s': %s\n", filename, error);
+    fail ("Error opening '%s': %s\n", filename, error->message);
 
   /* The files have ISO-8859-1 copyright signs in them */
   if (!g_io_channel_set_encoding (channel, "ISO-8859-1", &error))
-      fail ("Cannot set encoding when reading '%s': %s\n", filename, error);
+      fail ("Cannot set encoding when reading '%s': %s\n", filename, error->message);
   
   while (status == G_IO_STATUS_NORMAL)
     {
@@ -188,7 +203,7 @@ scripts_for_file (const char *base_dir,
     }
 
   if (!g_io_channel_shutdown (channel, FALSE, &error))
-    fail ("Error closing '%s': %s\n", channel, error);
+    fail ("Error closing '%s': %s\n", filename, error->message);
 
   g_free (filename);
 }
@@ -219,7 +234,7 @@ compare_lang (gconstpointer a,
 	      gconstpointer b)
 {
   const LangInfo *info_a = a;
-  const LangInfo *info_b = a;
+  const LangInfo *info_b = b;
 
   return strcmp (pango_language_to_string (info_a->lang),
 		 pango_language_to_string (info_b->lang));
@@ -230,7 +245,8 @@ int main (int argc, char **argv)
   GDir *dir;
   GError *error = NULL;
   GArray *script_array;
-  int i, j;
+  unsigned int i;
+  int j;
   int max_lang_len = 0;
 
   g_type_init ();
@@ -261,7 +277,7 @@ int main (int argc, char **argv)
       LangInfo *info = &g_array_index (script_array, LangInfo, i);
       
       max_lang_len = MAX (max_lang_len,
-			  1 + strlen (pango_language_to_string (info->lang)));
+			  1 + (int)strlen (pango_language_to_string (info->lang)));
     }
       
   g_print ("typedef struct {\n"
@@ -269,7 +285,7 @@ int main (int argc, char **argv)
 	   "  PangoScript scripts[%d];\n"
 	   "} PangoScriptForLang;\n"
 	   "\n"
-	   "PangoScriptForLang pango_script_for_lang[] = {\n",
+	   "static const PangoScriptForLang pango_script_for_lang[] = {\n",
 	   max_lang_len,
 	   MAX_SCRIPTS);
   
